/*
 * See the dyninst/COPYRIGHT file for copyright information.
 *
 * We provide the Paradyn Tools (below described as "Paradyn")
 * on an AS IS basis, and do not warrant its validity or performance.
 * We reserve the right to update, modify, or discontinue this
 * software at any time.  We shall have no obligation to supply such
 * updates or modifications or any other form of support to you.
 *
 * By your use of Paradyn, you understand and agree that we (or any
 * other person or entity with proprietary rights in Paradyn) are
 * under no obligation to provide either maintenance services,
 * update services, notices of latent defects, or correction of
 * defects for Paradyn.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
/* Public Interface */

#ifndef PATCHAPI_H_SNIPPET_H_
#define PATCHAPI_H_SNIPPET_H_

#include "PatchCommon.h"
#include "Point.h"
#include "Buffer.h"

namespace Dyninst
{
namespace PatchAPI
{
/* Interface for snippet representation. */

/*
  To extend Snippet:

    - Prepare a constructor and pass in whatever structures that are needed to
      generate code, e.g., codeGen in dyninst.
    - Implement generateCode().
 */
class PATCHAPI_EXPORT Snippet
{
public:
    typedef boost::shared_ptr<Snippet> Ptr;
    Snippet() {}
    virtual ~Snippet() {}
    static Snippet::Ptr create(Snippet* a) { return Ptr(a); }

    // Returns false if code generation failed catastrophically
    // Point is an in-param that identifies where the snippet is
    // being generated.
    // Buffer is an out-param that holds the generated code.
    virtual bool generate(Point*, Buffer&) = 0;
};

}  // namespace PatchAPI
}  // namespace Dyninst
#endif  // PATCHAPI_H_SNIPPET_H_
