# - Config file for omnitrace and its component libraries
# It defines the following variables:
#
# omnitrace_INCLUDE_DIRS
# omnitrace_LIBRARIES
# omnitrace_INTERNAL_DEFINES - used by the test suite

# compute paths
get_filename_component(omnitrace_CMAKE_DIR "${CMAKE_CURRENT_LIST_FILE}" PATH)

# version
include(${CMAKE_CURRENT_LIST_DIR}/omnitrace-version.cmake)


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was omnitrace-config.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################

set_and_check(omnitrace_INCLUDE_DIR "${PACKAGE_PREFIX_DIR}/include")
set_and_check(omnitrace_LIB_DIR "${PACKAGE_PREFIX_DIR}/lib")
get_filename_component(omnitrace_ROOT_DIR ${omnitrace_INCLUDE_DIR} PATH)

set(omnitrace_LIBRARIES)

add_library(omnitrace::omnitrace INTERFACE IMPORTED)

include("${omnitrace_CMAKE_DIR}/omnitrace-library-targets.cmake")

# Library dependencies
foreach(TARG user)
    set(TARG omnitrace-${TARG}-library)
    if(NOT omnitrace_FIND_COMPONENTS)
        list(APPEND omnitrace_LIBRARIES omnitrace::${TARG})
        target_link_libraries(omnitrace::omnitrace
                              INTERFACE omnitrace::${TARG})
    endif()
endforeach()

if(omnitrace_FIND_COMPONENTS)
    foreach(COMP ${omnitrace_FIND_COMPONENTS})
        set(TARG omnitrace::omnitrace-${COMP}-library)
        if(TARGET ${TARG})
            set(omnitrace_${COMP}_FOUND 1)
            list(APPEND omnitrace_LIBRARIES ${TARG})
            target_link_libraries(omnitrace::omnitrace
                                  INTERFACE ${TARG})
        else()
            set(omnitrace_${COMP}_FOUND 0)
        endif()
    endforeach()
endif()

include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(
    omnitrace
    FOUND_VAR omnitrace_FOUND
    REQUIRED_VARS omnitrace_ROOT_DIR omnitrace_INCLUDE_DIR omnitrace_LIBRARIES
    VERSION_VAR omnitrace_VERSION
    HANDLE_COMPONENTS)
